@@ -11,10 +11,11 @@
 #******************************************************************************************
 
 linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
-                    wght_thres = NULL, sort = NULL, Dom = NULL, 
-                    period = NULL, dataset = NULL, percentage = 60, 
-                    order_quant = 50, var_name = "lin_arpr") {
- 
+                    wght_thres = NULL, sort = NULL, Dom = NULL,
+                    period = NULL, dataset = NULL, percentage = 60,
+                    order_quant = 50, var_name = "lin_arpr",
+                    checking = TRUE) {
+
    ## initializations
    if (min(dim(data.table(var_name)) == 1) != 1) {
        stop("'var_name' must have defined one name of the linearized variable")}
@@ -29,130 +30,69 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
    if(length(oq) != 1 | any(!is.numeric(oq) | oq < 0 | oq > 100)) {
           stop("'order_quant' must be a numeric value in [0, 100]") }
 
-   if(!is.null(dataset)) {
-       dataset <- data.table(dataset)
-       if (checker(Y, dataset, "Y"))  Y <- dataset[, Y, with = FALSE] 
+   if (checking) {
+        Y <- check_var(vars = Y, varn = "Y", dataset = dataset,
+                       ncols = 1, isnumeric = TRUE,
+                       isvector = TRUE, grepls = "__")
+        Ynrow <- length(Y)
 
-       if(!is.null(id)) {
-          if (checker(id, dataset, "id")) id <- dataset[, id, with = FALSE]}
+        Y_thres <- check_var(vars = Y_thres, varn = "Y_thres",
+                             dataset = dataset, ncols = 1,
+                             Ynrow = Ynrow, mustbedefined = FALSE,
+                             isnumeric = TRUE, isvector = TRUE)
 
-       if(!is.null(weight)) {
-           if (checker(weight, dataset, "weight")) weight <- dataset[, weight, with = FALSE] }
+        weight <- check_var(vars = weight, varn = "weight",
+                            dataset = dataset, ncols = 1,
+                            Ynrow = Ynrow, isnumeric = TRUE,
+                            isvector = TRUE)
 
-       if(!is.null(Y_thres)) {
-           if (checker(Y_thres, dataset, "Y_thres")) Y_thres <- dataset[, Y_thres, with = FALSE] }
-     
-       if(!is.null(wght_thres)) {
-           if (checker(wght_thres, dataset, "wght_thres")) wght_thres <- dataset[, wght_thres, with = FALSE] }
+        wght_thres <- check_var(vars = wght_thres, varn = "wght_thres",
+                                dataset = dataset, ncols = 1,
+                                Ynrow = Ynrow, mustbedefined = FALSE,
+                                isnumeric = TRUE, isvector = TRUE)
 
-       if(!is.null(sort)) {
-           if (checker(sort, dataset, "sort")) sort <- dataset[, sort, with = FALSE] }
+        sort <- check_var(vars = sort, varn = "sort",
+                          dataset = dataset, ncols = 1,
+                          Ynrow = Ynrow, mustbedefined = FALSE,
+                          isnumeric = TRUE, isvector = TRUE)
 
-       if (!is.null(period)) {
-            if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
-            if (min(period %in% names(dataset))==1) period <- dataset[, period, with = FALSE]}
+        period <- check_var(vars = period, varn = "period",
+                            dataset = dataset, Ynrow = Ynrow,
+                            ischaracter = TRUE, mustbedefined = FALSE,
+                            duplicatednames = TRUE)
 
-       if (!is.null(Dom)) {
-            if (checker(Dom, dataset, "Dom")) Dom <- data.table(dataset[, Dom, with = FALSE])}
+        Dom <- check_var(vars = Dom, varn = "Dom", dataset = dataset,
+                         Ynrow = Ynrow, ischaracter = TRUE,
+                         mustbedefined = FALSE, duplicatednames = TRUE,
+                         grepls = "__")
+
+        id <- check_var(vars = id, varn = "id", dataset = dataset,
+                        ncols = 1, Ynrow = Ynrow, ischaracter = TRUE,
+                        periods = period)
     }
+   dataset <- NULL
 
-   # check vectors
-   # Y
-   Y <- data.frame(Y)
-   n <- nrow(Y)
-   if (anyNA(Y)) stop("'Y' has missing values")
-   if (ncol(Y) != 1) stop("'Y' must be a vector or 1 column data.frame, matrix, data.table")
-   Y <- Y[, 1]
-   if(!is.numeric(Y)) stop("'Y' must be numeric")
-
-   # weight
-   weight <- data.frame(weight)
-   if (anyNA(weight)) stop("'weight' has missing values")
-   if (nrow(weight) != n) stop("'weight' must be the same length as 'inc'")
-   if (ncol(weight) != 1) stop("'weight' must be a vector or 1 column data.frame, matrix, data.table")
-   weight <- weight[,1]
-   if (!is.numeric(weight)) stop("'weight' must be numeric")
-
-   # Y_thres
    if (is.null(Y_thres)) Y_thres <- Y
-   Y_thres <- data.frame(Y_thres)
-   if (anyNA(Y_thres)) stop("'Y_thres' has missing values") 
-   if (nrow(Y_thres) != n) stop("'Y_thres' must be the same length as 'Y'")
-   if (ncol(Y_thres) != 1) stop("'Y_thres' must be a vector or 1 column data.frame, matrix, data.table")
-   Y_thres <- Y_thres[, 1]
-   if(!is.numeric(Y_thres)) stop("'Y_thres' must be numeric")
-
-   # wght_thres
    if (is.null(wght_thres)) wght_thres <- weight
-   wght_thres <- data.frame(wght_thres)
-   if (anyNA(wght_thres)) stop("'wght_thres' has missing values") 
-   if (nrow(wght_thres) != n) stop("'wght_thres' must be the same length as 'Y'")
-   if (ncol(wght_thres) != 1) stop("'wght_thres' must be a vector or 1 column data.frame, matrix, data.table")
-   wght_thres <- wght_thres[, 1]
-   if (!is.numeric(wght_thres)) stop("'wght_thres' must be numeric")
-    
-   # sort
-   if (!is.null(sort)) {    
-        sort <- data.frame(sort)
-        if (anyNA(sort)) stop("'sort' has missing values")
-        if (length(sort) != n) stop("'sort' must have the same length as 'Y'")
-        if (ncol(sort) != 1) stop("'sort' must be a vector or 1 column data.frame, matrix, data.table")
-        sort <- sort[, 1]
-   }
 
-   # period     
-   if (!is.null(period)) {
-       period <- data.table(period)
-       if (any(duplicated(names(period)))) 
-                 stop("'period' are duplicate column names: ", 
-                      paste(names(period)[duplicated(names(period))], collapse = ","))
-       if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
-       period[, (names(period)) := lapply(.SD, as.character)]
-       if(anyNA(period)) stop("'period' has missing values")
-   }   
-   
-   # id
-   if (is.null(id)) id <- 1 : n
-   id <- data.table(id)
-   if (anyNA(id)) stop("'id' has missing values")
-   if (ncol(id) != 1) stop("'id' must be 1 column data.frame, matrix, data.table")
-   if (nrow(id) != n) stop("'id' must be the same length as 'Y'")
-   if (names(id) == "id") setnames(id, names(id), "ID")
-   if (is.null(period)){ if (any(duplicated(id))) stop("'id' are duplicate values") 
-                       } else {
-                          id1 <- data.table(period, id)
-                          if (any(duplicated(id1))) stop("'id' by period are duplicate values")
-                         }
 
-   # Dom     
-   if (!is.null(Dom)) {
-             Dom <- data.table(Dom)
-             if (any(duplicated(names(Dom)))) 
-                 stop("'Dom' are duplicate column names: ", 
-                      paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
-             if (nrow(Dom) != n) stop("'Dom' must be the same length as 'Y'")
-             Dom[, (names(Dom)) := lapply(.SD, as.character)] 
-             if (anyNA(Dom)) stop("'Dom' has missing values")
-             if (any(grepl("__", names(Dom)))) stop("'Dom' is not allowed column names with '__'")
-       }
+   ## computations
+   ind0 <- rep.int(1, length(Y))
+   period_agg <- period1 <- NULL
+   if (!is.null(period)) { period1 <- copy(period)
+                           period_agg <- data.table(unique(period))
+                       } else period1 <- data.table(ind = ind0)
+   period1_agg <- data.table(unique(period1))
 
-    ## computations
-    ind0 <- rep.int(1, n)
-    period_agg <- period1 <- NULL
-    if (!is.null(period)) { period1 <- copy(period)
-                            period_agg <- data.table(unique(period))
-                        } else period1 <- data.table(ind = ind0)
-    period1_agg <- data.table(unique(period1))
+   # ARPR by domain (if requested)
+   quantile <- incPercentile(Y = Y_thres,
+                             weights = wght_thres,
+                             sort = sort, Dom = NULL,
+                             period = period,
+                             k = order_quant,
+                             dataset = NULL,
+                             checking = FALSE)
 
-    # ARPR by domain (if requested)
-    quantile <- incPercentile(Y = Y_thres,
-                              weights = wght_thres,
-                              sort = sort, Dom = NULL,
-                              period = period,
-                              k = order_quant,
-                              dataset = NULL)
-
-    quantile <- data.table(quantile)
     setnames(quantile, names(quantile)[ncol(quantile)], "quantile")
     if (ncol(quantile) > 1) setkeyv(quantile, head(names(quantile), -1))
     threshold <- copy(quantile)
@@ -165,7 +105,7 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
     if (!is.null(Dom)) {
         Dom_agg <- data.table(unique(Dom))
         setkeyv(Dom_agg, names(Dom_agg))
-          
+
         arpr_v <- c()
         arpr_m <- copy(arpr_id)
         for(i in 1 : nrow(Dom_agg)) {
@@ -174,7 +114,7 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
               ind <- as.integer(rowSums(Dom == Dom_agg[i,][ind0,]) == ncol(Dom))
 
               arprl <- lapply(1 : nrow(period1_agg), function(j) {
-                               if (!is.null(period)) { 
+                               if (!is.null(period)) {
                                        rown <- cbind(period_agg[j], Dom_agg[i])
                                        setkeyv(rown, names(rown))
                                        rown2 <- copy(rown)
@@ -183,7 +123,7 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
                                              rown2 <- Dom_agg[i] }
 
                                indj <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
-      
+
                                arpr_l <- arprlinCalc(Y1 = Y[indj],
                                                      ids = arpr_id[indj],
                                                      wght1 = weight[indj],
@@ -200,16 +140,16 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
 
                  setnames(arprlin, names(arprlin), c(names(arpr_id), var_nams))
                  arpr_m <- merge(arpr_m, arprlin, all.x = TRUE, by = names(arpr_id))
-                 arpr_v <- rbind(arpr_v, arprs) 
+                 arpr_v <- rbind(arpr_v, arprs)
            }
      } else { arprl <- lapply(1:nrow(period1_agg), function(j) {
-                           if (!is.null(period)) { 
+                           if (!is.null(period)) {
                                          rown <- period_agg[j]
                                          rown <- merge(rown, quantile, all.x = TRUE,
                                                        by = names(rown))
                                        } else rown <- quantile
                            ind2 <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
-      
+
                            arpr_l <- arprlinCalc(Y1 = Y[ind2],
                                                  ids = arpr_id[ind2],
                                                  wght1 = weight[ind2],
@@ -218,8 +158,8 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
                                                  wght_thresh = wght_thres[ind2],
                                                  percent = p,
                                                  order_quants = order_quant,
-                                                 quant_val = rown[["quantile"]]) 
-                          if (!is.null(period)) { 
+                                                 quant_val = rown[["quantile"]])
+                          if (!is.null(period)) {
                                    arprs <- data.table(period_agg[j], arpr = arpr_l$rate_val_pr)
                              } else arprs <- data.table(arpr = arpr_l$rate_val_pr)
                           list(arpr = arprs, lin = arpr_l$lin)
@@ -227,7 +167,7 @@ linarpr <- function(Y, id = NULL, weight = NULL, Y_thres = NULL,
                arpr_v <- rbindlist(lapply(arprl, function(x) x[[1]]))
                arpr_m <- rbindlist(lapply(arprl, function(x) x[[2]]))
                setnames(arpr_m, names(arpr_m), c(names(arpr_id), var_name))
-            } 
+            }
      arpr_m[is.na(arpr_m)] <- 0
      setkeyv(arpr_m, names(arpr_id))
      return(list(quantile = quantile, threshold = threshold, value = arpr_v, lin = arpr_m))
@@ -241,17 +181,17 @@ arprlinCalc <- function(Y1, ids, wght1, indicator, Y_thresh, wght_thresh, percen
 
     inc2 <- Y_thresh
     wght2 <- wght_thresh
-  
+
     wt <- indicator * wght1
     thres_val <- percent / 100 * quant_val
-    N0 <- sum(wght2)   # Estimated whole population size 
+    N0 <- sum(wght2)   # Estimated whole population size
     N <- sum(wt)       # Estimated (sub)population size
 
     poor <- (Y1 <= thres_val)
     rate_val <- sum(wt * poor) / N  # Estimated poverty rate */
     rate_val_pr <- 100 * rate_val
 
-    h <- sqrt((sum(wght2 * inc2 * inc2) - sum(wght2 * inc2) * sum(wght2 * inc2) / sum(wght2)) / sum(wght2)) / exp(0.2 * log(sum(wght2))) 
+    h <- sqrt((sum(wght2 * inc2 * inc2) - sum(wght2 * inc2) * sum(wght2 * inc2) / sum(wght2)) / sum(wght2)) / exp(0.2 * log(sum(wght2)))
     # h=S/N^(1/5)
 
     #---- 1. Linearization of the poverty threshold ----
